//	74.442	Advanced Design & Analysis of Algorithms
//	Spring 1998
//	Dr. Bill Kocay
//
//	Hamish Carr
//	Project:	Toroidal Surface Geometries
//
//

#include "Torus.h"

Tile *Torus::FindTile(BPoint where)											//	finds the tile corresponding with a point
	{
	Tile **walker;															//	for walking along the array
	Tile **start, **stop;													//	sentinels for walk loop
	long left, right, middle;												//	indices for start, stop
	
	left = 0; right = nTiles -1;											//	indices for search

	start = sortedTiles;													//	start at the beginning of the sorted list
	stop = start + right;													//	set the sentinel

	//	start off with a binary search for the first tile with positive distance
	
	if ((**stop).position.z <= 0)											//	if all points are behind us
		return NULL;														//	return no tile
	
	if ((**start).position.z > 0)											//	if all points are ahead of us
		walker = start;														//	we start at the beginning	
	else // start is nonpositive
		{
		for ( ; ; )															//	loop until we bail out
			{
			if (start == stop - 1)											//	if they are only one step apart
				break;														//	we're done with the loop
			middle = (left + right)/2;										//	find the midpoint
			walker = sortedTiles + middle;									//	find the next tile
			if ((**walker).position.z > 0)									//	overshoot
				{
				stop = walker;												//	use it as new right end
				right = middle;												//	and reset the index
				}
			else 															//	otherwise
				{
				start = walker;												//	use it as new left end
				left = middle;												//	and reset the index
				} // end of test for position
			} // end of for loop	
		walker = stop;														//	start at the first positive one
		} // end of nonpositive case
	
	stop = start + nTiles;													//	use stop as the sentinel
	
	//	OK, we are at the start of positive distance tiles.  We do a linear search on the rest
	for ( ; walker < stop; walker++)										//	walk along the list
		{
		if ((**walker).Contains(where))										//	if the tile contains the point
			return *walker;													//	return a pointer to it
		} // end of loop along list
	return NULL;															//	we ran off the end - we didn't find it
	} // end of FindTile()
	
bool Torus::MakePoint(Tile *target)											//	make the point corresponding to the tile
	{
	selectedPoints[nPointsSelected] = target;								//	add it to the selection
	nPointsSelected++;														//	and increment the # selected
	target->isSelected = true;												//	mark it as selected

	target->hasPoint = true;												//	toggle the existence of the point
	thePoints[nPoints] = target;											//	and add it to the array of points
	nPoints++;																//	increment the number of points
	return true;															//	and advise the view to update
	} // end of MakePoint()

bool Torus::DeletePoint(Tile *target)										//	delete the specified point
	{
	TilePointer *seek, *seek2;												//	for searching along array of points

	target->hasPoint = false;												//	toggle the existence of the point
	nPoints--;																//	decrement the number of points
	for (seek = thePoints; *seek != target; seek++){}						//	walk along the array to the target
	if ((*seek)->isSelected)												//	if the point is currently selected
		{
		for (seek2 = selectedPoints; seek2 < selectedPoints + nPointsSelected; seek2++)
			if (*seek2 == *seek) break;										//	we've found it in the selection list
		if (seek2 < selectedPoints + nPointsSelected)						//	if we found it
			{
			nPointsSelected--;												//	take one point of the selection array
			*seek2 = selectedPoints[nPointsSelected];						//	swap the last one into its place
			(*seek)->isSelected = false;									//	mark it as unselected on the tile
			selectedPoints[nPointsSelected] = NULL;							//	and set the end one to NULL
			} // end of test for found it in selection array
		} // end of selection loop
	*seek = thePoints[nPoints];												//	swap the end tile to it's position 
	thePoints[nPoints] = NULL;												//	and nullify the end pointer
	return true;															//	advise the view to update
	} // end of DeletePoint()

bool Torus::TogglePoint(BPoint where, bool shiftIsDown)						//	toggle the existence of a point
																			//	returns true if update needed
	{
	Tile *target;															//	the tile targetted by the click

	target = FindTile(where);												//	find which tile
	if (target == NULL) return false;										//	we didn't find any
	if (target->hasPoint)													//	if the point exists
		{
		return DeletePoint(target);											//	delete the point
		} // end of case where point exists
	else // i.e. point doesn't exist yet
		{
		if (nPoints >= maxNPoints)												//	if there are more points than permitted
			{
			char message[100];													//	message to print
			sprintf(message, "Sorry.  I am only allowed %d points.", nPoints);	//	set up the error message
			DoAlert(message);													//	pop up an alert box
			return false;														//	and advise the view not to update
			} // end of case where too many points
		else																	//	i.e. we have room for another point
			{
			if (! shiftIsDown)													//	if the shift key isn't down
				ClearPointSelection();											//	clear all selected points
			return MakePoint(target);											//	create the point
			} // end of case where point is to be added
		} // end of case where point doesn't exist yet
	} // end of TogglePoint()

bool Torus::TogglePointSelection(BPoint where, bool shiftIsDown)			//	toggle whether a given point is selected
	{
	Tile *target;															//	the tile targetted by the click
	bool changesMade = false;												//	whether we have made changes
	TilePointer *seek;														//	pointer for finding it in selection array

	target = FindTile(where);												//	find which tile was hit

	if (!shiftIsDown)														//	if the shift key isn't down
		changesMade = ClearPointSelection();								//	clear the selection

	if (target == NULL)														//	if there was no tile there
		return changesMade;													//	say whether we cleared the selection

	if (!target->hasPoint)													//	if there was no point on the tile
		return changesMade;													//	say whether we cleared the selection
		
	if (target->isSelected)													//	if the point was already selected		
		{
		for (seek = selectedPoints; seek < selectedPoints + nPointsSelected; seek++)
			if (*seek == target) break;										//	we've found it in the selection list
		if (seek < selectedPoints + nPointsSelected)						//	if we found it
			{
			nPointsSelected--;												//	take one point of the selection array
			target->isSelected = false;										//	mark it as unselected
			*seek = selectedPoints[nPointsSelected];						//	swap the last one into its place
			selectedPoints[nPointsSelected] = NULL;							//	and set the end one to NULL
			} // end of test for found it in selection array
		} // end of case of previously selected point
	else																	//	i.e. point is unselected
		{
		target->isSelected = true;											//	mark it as selected
		selectedPoints[nPointsSelected] = target;							//	add it to the selection array
		nPointsSelected++;													//	and increment the points selected
		} // end of case of unselected point
	
	return true;															//	and say we changed things
	} // end of TogglePointSelection()

bool Torus::ClearPointSelection()											//	clears the selection array
	{
	long i;																	//	index
	
	for (i = 0; i < nPointsSelected; i++)									//	walk along selection array
		{
		selectedPoints[i]->isSelected = false;								//	switch off the selection
		selectedPoints[i] = NULL;											//	and point the selection at nothing
		} // end of loop along selection array
	nPointsSelected = 0;													//	reset the # of selected points
	return (i > 0);															//	return whether we changed any points
	} // end of ClearPointSelection()
