//	74.442	Advanced Analysis of Algorithms
//	Spring, 1998
//	Dr. Kocay
//
//	Assignment 3a:  Mandelbrot Set
//
//	Hamish Carr	

#include "TorusSettingsWindow.h"

#define textTorus "&Torus\nUntitled Torus\nTube Radius:\n%f\nRing Radius:\n%f\n# Phi Divisions:\n%d\n# Theta Divisions\n%d\n"\
				  "# Points\n0\n# Lines\n0\n"

TorusSettingsWindow::TorusSettingsWindow()									//	constructor
	: BWindow(BRect(100, 100, 400, 400), "New Torus ...", B_TITLED_WINDOW, 
		B_NOT_RESIZABLE | B_NOT_ZOOMABLE | B_NOT_MINIMIZABLE)				//	inherited constructor
	{
	thetaEntry = new BTextControl(BRect(50, 25, 250, 50), "# of Theta Divisions",
									"# of Theta Divisions", "120",
									new BMessage(TSW_THETA_SET));			//	create the theta box
	thetaEntry->SetDivider(150);											//	make it 3/4 text
	AddChild(thetaEntry);													//	add the text box to the window

	phiEntry = new BTextControl(BRect(50, 75, 250, 100), "# of Phi Divisions",
									"# of Phi Divisions", "24",
									new BMessage(TSW_PHI_SET));
	phiEntry->SetDivider(150);												//	make it 3/4 text
	AddChild(phiEntry);														//	add the text box to the window

	ringRadEntry = new BTextControl(BRect(50, 125, 250, 150), "Ring Radius",
									"Ring Radius", "2000.00",
									new BMessage(TSW_RINGRAD_SET));			//	create the theta box
	ringRadEntry->SetDivider(150);											//	make it 3/4 text
	AddChild(ringRadEntry);													//	add the text box to the window

	tubeRadEntry = new BTextControl(BRect(50, 175, 250, 200), "Tube Radius",
									"Tube Radius", "1000.00",
									new BMessage(TSW_TUBERAD_SET));
	tubeRadEntry->SetDivider(150);											//	make it 3/4 text
	AddChild(tubeRadEntry);													//	add the text box to the window

	thetaDivs = 120;														//	set the initial values
	phiDivs = 24;		
	
	ringRad = 2000;															//	set default radius for ring
	tubeRad = 1000;															//	set default radius for tube
	
	helpButton = new BButton(BRect(50, 225, 100, 250), "Help! I'm trapped", "Help",
									new BMessage(TSW_HELP_WANTED));			//	create the help button
	AddChild(helpButton);													//	add it to the window
	
	cancelButton = new BButton(BRect(125, 225, 175, 250), "Get me out of here!", "Cancel",
									new BMessage(TSW_CANCEL_IT));			//	create the cancel button
	AddChild(cancelButton);													//	add it to the window

	OKButton = new BButton(BRect(200, 225, 250, 250), "Go ahead, see if I care", "OK",
									new BMessage(TSW_DO_IT));				//	create the OK button
	AddChild(OKButton);														//	add it to the window

	OKButton->MakeFocus();													//	set the OK button to be the default
	} // end of TorusSettingsWindow constructor

TorusSettingsWindow::~TorusSettingsWindow()									//	destructor
	{
	} // end of TorusSettingsWindow destructor
		
void TorusSettingsWindow::MessageReceived(BMessage *theEvent)				//	processes messages
	{
	switch(theEvent->what)													//	yer basic case statement
		{
		case TSW_THETA_SET:													//	the theta field has been changed
			ResetTheta();													//	call the function to do it
			break;
		case TSW_PHI_SET:													//	the phi field has been changed
			ResetPhi();														//	call the function to do it
			break;
		case TSW_RINGRAD_SET:												//	the ringradius field has been changed
			ResetRingRad();													//	call the function to do it
			break;
		case TSW_TUBERAD_SET:												//	the phi field has been changed
			ResetTubeRad();													//	call the function to do it
			break;
		case TSW_HELP_WANTED:												//	the user has asked for help
			BAlert *helpWindow = new BAlert("Torus Settings Help", TSW_HELP_MESSAGE, "Done", NULL, NULL, 
							B_WIDTH_FROM_LABEL,B_STOP_ALERT);				//	set the alert up
			helpWindow->Go();												//	run it & wait for it to return
			break;
		case TSW_CANCEL_IT:													//	the user asked to cancel
			long nWindows;													//	how many windows the app has
			nWindows = be_app->CountWindows();								//	find out how many
			if (nWindows <= 2)												//	i.e. this is the last window other than the save panel
				be_app->PostMessage(B_QUIT_REQUESTED);						//	tell the app we would like to quit
			Hide();															//	hide the window for next time
			break;							
		case TSW_DO_IT:														//	the user asked to create a new window
			ResetTheta();													//	update all the parameter fields
			ResetPhi();									
			ResetRingRad();								
			ResetTubeRad();								
			char *newTorus = new char[1024];								//	create a buffer for a new torus message
			sprintf(newTorus, textTorus, tubeRad, ringRad, phiDivs, thetaDivs);	
																			//	create the text version
			TorusWindow *aTorusWindow = new TorusWindow(defaultTorusWindowRect, newTorus);
																			//	create a window for the torus
			aTorusWindow->Show();											//	and show it
			Hide();															//	hide until the next time
			break;	
		default:
			BWindow::MessageReceived(theEvent);								//	pass it on to the BWindow class
			break;
		} // end of switch statement	
	} // end of MessageReceived()
	
void TorusSettingsWindow::ResetTheta()										//	reset the theta value based on contents of box
	{
	char newValBuf[10];														//	buffer for new value
	
	long newTheta = atol(thetaEntry->Text());								//	retrieve it as a long integer
	if (newTheta < TSW_MIN_THETAS) newTheta = TSW_MIN_THETAS;				//	truncate it low
	else if (newTheta > TSW_MAX_THETAS) newTheta = TSW_MAX_THETAS;			//	truncate it high
	thetaDivs = newTheta;													//	update it in the window
	sprintf(newValBuf, "%d", thetaDivs);									//	create a suitable string
	thetaEntry->SetText(newValBuf);											//	and set the theta box' entry			
	} // end of ResetTheta()
	
void TorusSettingsWindow::ResetPhi()										//	ditto for phi
	{
	char newValBuf[10];														//	buffer for new value
	
	long newPhi = atol(phiEntry->Text());									//	retrieve it as a long integer
			
	if (newPhi < TSW_MIN_PHIS) newPhi = TSW_MIN_PHIS;						//	truncate it low
	else if (newPhi > TSW_MAX_PHIS) newPhi = TSW_MAX_PHIS;					//	truncate it high
	phiDivs = newPhi;														//	update it in the window
	sprintf(newValBuf, "%d", phiDivs);										//	create a suitable string
	phiEntry->SetText(newValBuf);											//	and set the phi box' entry			
	} // end of ResetPhi()

void TorusSettingsWindow::ResetRingRad()									//	do it for ring radius
	{
	char newValBuf[10];														//	buffer for new value
	float newRingRad = atof(ringRadEntry->Text());							//	retrieve it as a float
			
	if (newRingRad < TSW_MIN_RINGRAD) newRingRad = TSW_MIN_RINGRAD;			//	truncate it low
	else if (newRingRad > TSW_MAX_RINGRAD) newRingRad = TSW_MAX_RINGRAD;	//	truncate it high

//	if (newRingRad < tubeRad)												//	make sure that it is bigger than the tube radius
//		newRingRad = tubeRad;
	ringRad = newRingRad;													//	update it in the window
	sprintf(newValBuf, "%f.2", ringRad);									//	create a suitable string
	ringRadEntry->SetText(newValBuf);										//	and set the phi box' entry			
	} // end of ResetRingRad()

void TorusSettingsWindow::ResetTubeRad()									//	and, inevitably, for tube radius
	{
	char newValBuf[10];														//	buffer for new value
	float newTubeRad = atof(tubeRadEntry->Text());							//	retrieve it as a float
			
	if (newTubeRad < TSW_MIN_TUBERAD) newTubeRad = TSW_MIN_TUBERAD;			//	truncate it low
	else if (newTubeRad > TSW_MAX_TUBERAD) newTubeRad = TSW_MAX_TUBERAD;	//	truncate it high

//	if (newTubeRad > ringRad)												//	make sure that it is smaller than the ring radius
//		newTubeRad = ringRad;
	tubeRad = newTubeRad;													//	update it in the window
	sprintf(newValBuf, "%f.2", tubeRad);									//	create a suitable string
	tubeRadEntry->SetText(newValBuf);										//	and set the phi box' entry			
	} // end of ResetTubeRad()
	
	